@@ -2,13 +2,13 @@ use std::cell::RefCell;
 use std::task::{Context, Poll};
 use std::{future::Future, marker::PhantomData, num::NonZeroU16, pin::Pin, rc::Rc};
 
+use ntex::io::DispatchItem;
 use ntex::service::{fn_factory_with_config, Service, ServiceFactory};
 use ntex::util::{
     buffer::BufferService, inflight::InFlightService, join, Either, HashSet, Ready,
 };
 
 use crate::error::{MqttError, ProtocolError};
-use crate::io::DispatchItem;
 
 use super::control::{
     ControlMessage, ControlResult, ControlResultKind, Subscribe, Unsubscribe,
@@ -22,29 +22,17 @@ pub(super) fn factory<St, T, C, E>(
     control: C,
     inflight: usize,
 ) -> impl ServiceFactory<
-    Config = Session<St>,
-    Request = DispatchItem<Rc<MqttShared>>,
+    DispatchItem<Rc<MqttShared>>,
+    Session<St>,
     Response = Option<codec::Packet>,
     Error = MqttError<E>,
     InitError = MqttError<E>,
 >
 where
-    E: 'static,
     St: 'static,
-    T: ServiceFactory<
-            Config = Session<St>,
-            Request = Publish,
-            Response = (),
-            Error = E,
-            InitError = MqttError<E>,
-        > + 'static,
-    C: ServiceFactory<
-            Config = Session<St>,
-            Request = ControlMessage<E>,
-            Response = ControlResult,
-            Error = E,
-            InitError = MqttError<E>,
-        > + 'static,
+    T: ServiceFactory<Publish, Session<St>, Response = ()> + 'static,
+    C: ServiceFactory<ControlMessage<E>, Session<St>, Response = ControlResult> + 'static,
+    E: From<C::Error> + From<C::InitError> + From<T::Error> + From<T::InitError> + 'static,
 {
     fn_factory_with_config(move |cfg: Session<St>| {
         // create services
@@ -52,19 +40,23 @@ where
 
         async move {
             let (publish, control) = fut.await;
+            let publish = publish.map_err(|e| MqttError::Service(e.into()))?;
+            let control = control
+                .map_err(|e| MqttError::Service(e.into()))?
+                .map_err(|e| MqttError::Service(E::from(e)));
 
             let control = BufferService::new(
                 16,
-                || MqttError::<C::Error>::Disconnected,
+                || MqttError::<E>::Disconnected(None),
                 // limit number of in-flight messages
-                InFlightService::new(1, control?.map_err(MqttError::Service)),
+                InFlightService::new(1, control),
             );
 
             Ok(
                 // limit number of in-flight messages
                 InFlightService::new(
                     inflight,
-                    Dispatcher::<_, _, _, E>::new(cfg, publish?, control),
+                    Dispatcher::<_, _, _, E>::new(cfg, publish, control),
                 ),
             )
         }
@@ -72,7 +64,7 @@ where
 }
 
 /// Mqtt protocol dispatcher
-pub(crate) struct Dispatcher<St, T, C: Service, E> {
+pub(crate) struct Dispatcher<St, T, C: Service<ControlMessage<E>>, E> {
     session: Session<St>,
     publish: T,
     shutdown: RefCell<Option<Pin<Box<C::Future>>>>,
@@ -88,8 +80,9 @@ struct Inner<C> {
 
 impl<St, T, C, E> Dispatcher<St, T, C, E>
 where
-    T: Service<Request = Publish, Response = (), Error = E>,
-    C: Service<Request = ControlMessage<E>, Response = ControlResult, Error = MqttError<E>>,
+    E: From<T::Error>,
+    T: Service<Publish, Response = ()>,
+    C: Service<ControlMessage<E>, Response = ControlResult, Error = MqttError<E>>,
 {
     pub(crate) fn new(session: Session<St>, publish: T, control: C) -> Self {
         let sink = session.sink().clone();
@@ -104,14 +97,12 @@ where
     }
 }
 
-impl<St, T, C, E> Service for Dispatcher<St, T, C, E>
+impl<St, T, C, E> Service<DispatchItem<Rc<MqttShared>>> for Dispatcher<St, T, C, E>
 where
-    T: Service<Request = Publish, Response = (), Error = E>,
-    C: Service<Request = ControlMessage<E>, Response = ControlResult, Error = MqttError<E>>,
-    C::Future: 'static,
-    E: 'static,
+    E: From<T::Error> + 'static,
+    T: Service<Publish, Response = ()>,
+    C: Service<ControlMessage<E>, Response = ControlResult, Error = MqttError<E>> + 'static,
 {
-    type Request = DispatchItem<Rc<MqttShared>>;
     type Response = Option<codec::Packet>;
     type Error = MqttError<E>;
     type Future = Either<
@@ -120,7 +111,7 @@ where
     >;
 
     fn poll_ready(&self, cx: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
-        let res1 = self.publish.poll_ready(cx).map_err(MqttError::Service)?;
+        let res1 = self.publish.poll_ready(cx).map_err(|e| MqttError::Service(e.into()))?;
         let res2 = self.inner.control.poll_ready(cx)?;
 
         if res1.is_pending() || res2.is_pending() {
@@ -235,10 +226,9 @@ where
                     &self.inner,
                 )))
             }
-            DispatchItem::IoError(err) => Either::Right(Either::Right(ControlResponse::new(
-                ControlMessage::proto_error(ProtocolError::Io(err)),
-                &self.inner,
-            ))),
+            DispatchItem::Disconnect(err) => Either::Right(Either::Right(
+                ControlResponse::new(ControlMessage::peer_gone(err), &self.inner),
+            )),
             DispatchItem::WBackPressureEnabled | DispatchItem::WBackPressureDisabled => {
                 Either::Right(Either::Left(Ready::Ok(None)))
             }
@@ -248,7 +238,7 @@ where
 
 pin_project_lite::pin_project! {
     /// Publish service response future
-    pub(crate) struct PublishResponse<T: Service, C: Service, E> {
+    pub(crate) struct PublishResponse<T: Service<Publish>, C: Service<ControlMessage<E>>, E> {
         #[pin]
         state: PublishResponseState<T, C, E>,
         packet_id: Option<NonZeroU16>,
@@ -258,7 +248,7 @@ pin_project_lite::pin_project! {
 
 pin_project_lite::pin_project! {
     #[project = PublishResponseStateProject]
-    enum PublishResponseState<T: Service, C: Service, E> {
+    enum PublishResponseState<T: Service<Publish>, C: Service<ControlMessage<E>>, E> {
         Publish { #[pin] fut: T::Future },
         Control { #[pin] fut: ControlResponse<C, E> },
     }
@@ -266,8 +256,9 @@ pin_project_lite::pin_project! {
 
 impl<T, C, E> Future for PublishResponse<T, C, E>
 where
-    T: Service<Request = Publish, Response = (), Error = E>,
-    C: Service<Request = ControlMessage<E>, Response = ControlResult, Error = MqttError<E>>,
+    E: From<T::Error>,
+    T: Service<Publish, Response = ()>,
+    C: Service<ControlMessage<E>, Response = ControlResult, Error = MqttError<E>>,
 {
     type Output = Result<Option<codec::Packet>, MqttError<E>>;
 
@@ -290,7 +281,7 @@ where
                 }
                 Poll::Ready(Err(e)) => {
                     this.state.set(PublishResponseState::Control {
-                        fut: ControlResponse::new(ControlMessage::error(e), this.inner),
+                        fut: ControlResponse::new(ControlMessage::error(e.into()), this.inner),
                     });
                     self.poll(cx)
                 }
@@ -303,7 +294,7 @@ where
 
 pin_project_lite::pin_project! {
     /// Control service response future
-    pub(crate) struct ControlResponse<C: Service, E>
+    pub(crate) struct ControlResponse<C: Service<ControlMessage<E>>, E>
     {
         #[pin]
         fut: C::Future,
@@ -313,9 +304,9 @@ pin_project_lite::pin_project! {
     }
 }
 
-impl<C: Service, E> ControlResponse<C, E>
+impl<C, E> ControlResponse<C, E>
 where
-    C: Service<Request = ControlMessage<E>, Response = ControlResult, Error = MqttError<E>>,
+    C: Service<ControlMessage<E>, Response = ControlResult, Error = MqttError<E>>,
 {
     #[allow(clippy::match_like_matches_macro)]
     fn new(pkt: ControlMessage<E>, inner: &Rc<Inner<C>>) -> Self {
@@ -330,7 +321,7 @@ where
 
 impl<C, E> Future for ControlResponse<C, E>
 where
-    C: Service<Request = ControlMessage<E>, Response = ControlResult, Error = MqttError<E>>,
+    C: Service<ControlMessage<E>, Response = ControlResult, Error = MqttError<E>>,
 {
     type Output = Result<Option<codec::Packet>, MqttError<E>>;
 
