@@ -1,16 +1,15 @@
-use std::time::Instant;
 use std::{
     cell::RefCell, convert::TryFrom, fmt, future::Future, marker, num::NonZeroU16, rc::Rc,
 };
 
-use ntex::codec::{AsyncRead, AsyncWrite};
+use ntex::io::IoBoxed;
 use ntex::router::{IntoPattern, Path, Router, RouterBuilder};
 use ntex::service::{boxed, into_service, IntoService, Service};
 use ntex::time::{sleep, Millis, Seconds};
 use ntex::util::{ByteString, Either, HashMap, Ready};
 
 use crate::error::MqttError;
-use crate::io::{Dispatcher, Timer};
+use crate::io::Dispatcher;
 use crate::v5::publish::{Publish, PublishAck};
 use crate::v5::{codec, shared::MqttShared, sink::MqttSink, ControlResult};
 
@@ -18,8 +17,8 @@ use super::control::ControlMessage;
 use super::dispatcher::create_dispatcher;
 
 /// Mqtt client
-pub struct Client<Io> {
-    io: Io,
+pub struct Client {
+    io: IoBoxed,
     shared: Rc<MqttShared>,
     keepalive: Seconds,
     disconnect_timeout: Seconds,
@@ -27,7 +26,7 @@ pub struct Client<Io> {
     pkt: Box<codec::ConnectAck>,
 }
 
-impl<Io> fmt::Debug for Client<Io> {
+impl fmt::Debug for Client {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("v5::Client")
             .field("keepalive", &self.keepalive)
@@ -38,13 +37,10 @@ impl<Io> fmt::Debug for Client<Io> {
     }
 }
 
-impl<T> Client<T>
-where
-    T: AsyncRead + AsyncWrite + Unpin,
-{
+impl Client {
     /// Construct new `Dispatcher` instance with outgoing messages stream.
     pub(super) fn new(
-        io: T,
+        io: IoBoxed,
         shared: Rc<MqttShared>,
         pkt: Box<codec::ConnectAck>,
         max_receive: u16,
@@ -62,10 +58,7 @@ where
     }
 }
 
-impl<Io> Client<Io>
-where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
-{
+impl Client {
     #[inline]
     /// Get client sink
     pub fn sink(&self) -> MqttSink {
@@ -91,11 +84,11 @@ where
     }
 
     /// Configure mqtt resource for a specific topic
-    pub fn resource<T, F, U, E>(self, address: T, service: F) -> ClientRouter<Io, E, U::Error>
+    pub fn resource<T, F, U, E>(self, address: T, service: F) -> ClientRouter<E, U::Error>
     where
         T: IntoPattern,
-        F: IntoService<U>,
-        U: Service<Request = Publish, Response = PublishAck> + 'static,
+        F: IntoService<U, Publish>,
+        U: Service<Publish, Response = PublishAck> + 'static,
         E: From<U::Error>,
         PublishAck: TryFrom<U::Error, Error = E>,
     {
@@ -133,24 +126,18 @@ where
             }),
         );
 
-        let _ = Dispatcher::with(
-            self.io,
-            self.shared.state.clone(),
-            self.shared,
-            dispatcher,
-            Timer::new(Millis::ONE_SEC),
-        )
-        .keepalive_timeout(Seconds::ZERO)
-        .disconnect_timeout(self.disconnect_timeout)
-        .await;
+        let _ = Dispatcher::new(self.io, self.shared, dispatcher)
+            .keepalive_timeout(Seconds::ZERO)
+            .disconnect_timeout(self.disconnect_timeout)
+            .await;
     }
 
     /// Run client with provided control messages handler
     pub async fn start<F, S, E>(self, service: F) -> Result<(), MqttError<E>>
     where
         E: 'static,
-        F: IntoService<S> + 'static,
-        S: Service<Request = ControlMessage<E>, Response = ControlResult, Error = E> + 'static,
+        F: IntoService<S, ControlMessage<E>> + 'static,
+        S: Service<ControlMessage<E>, Response = ControlResult, Error = E> + 'static,
     {
         if self.keepalive.non_zero() {
             ntex::rt::spawn(keepalive(MqttSink::new(self.shared.clone()), self.keepalive));
@@ -164,26 +151,20 @@ where
             service.into_service(),
         );
 
-        Dispatcher::with(
-            self.io,
-            self.shared.state.clone(),
-            self.shared,
-            dispatcher,
-            Timer::new(Millis::ONE_SEC),
-        )
-        .keepalive_timeout(Seconds::ZERO)
-        .disconnect_timeout(self.disconnect_timeout)
-        .await
+        Dispatcher::new(self.io, self.shared, dispatcher)
+            .keepalive_timeout(Seconds::ZERO)
+            .disconnect_timeout(self.disconnect_timeout)
+            .await
     }
 }
 
 type Handler<E> = boxed::BoxService<Publish, PublishAck, E>;
 
 /// Mqtt client with routing capabilities
-pub struct ClientRouter<Io, Err, PErr> {
+pub struct ClientRouter<Err, PErr> {
+    io: IoBoxed,
     builder: RouterBuilder<usize>,
     handlers: Vec<Handler<PErr>>,
-    io: Io,
     shared: Rc<MqttShared>,
     keepalive: Seconds,
     disconnect_timeout: Seconds,
@@ -191,7 +172,7 @@ pub struct ClientRouter<Io, Err, PErr> {
     _t: marker::PhantomData<Err>,
 }
 
-impl<Io, Err, PErr> fmt::Debug for ClientRouter<Io, Err, PErr> {
+impl<Err, PErr> fmt::Debug for ClientRouter<Err, PErr> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("v5::ClientRouter")
             .field("keepalive", &self.keepalive)
@@ -201,9 +182,8 @@ impl<Io, Err, PErr> fmt::Debug for ClientRouter<Io, Err, PErr> {
     }
 }
 
-impl<Io, Err, PErr> ClientRouter<Io, Err, PErr>
+impl<Err, PErr> ClientRouter<Err, PErr>
 where
-    Io: AsyncRead + AsyncWrite + Unpin + 'static,
     Err: From<PErr> + 'static,
     PublishAck: TryFrom<PErr, Error = Err>,
     PErr: 'static,
@@ -212,8 +192,8 @@ where
     pub fn resource<T, F, S>(mut self, address: T, service: F) -> Self
     where
         T: IntoPattern,
-        F: IntoService<S>,
-        S: Service<Request = Publish, Response = PublishAck, Error = PErr> + 'static,
+        F: IntoService<S, Publish>,
+        S: Service<Publish, Response = PublishAck, Error = PErr> + 'static,
     {
         self.builder.path(address, self.handlers.len());
         self.handlers.push(boxed::service(service.into_service()));
@@ -236,24 +216,17 @@ where
             }),
         );
 
-        let _ = Dispatcher::with(
-            self.io,
-            self.shared.state.clone(),
-            self.shared,
-            dispatcher,
-            Timer::new(Millis::ONE_SEC),
-        )
-        .keepalive_timeout(Seconds::ZERO)
-        .disconnect_timeout(self.disconnect_timeout)
-        .await;
+        let _ = Dispatcher::new(self.io, self.shared, dispatcher)
+            .keepalive_timeout(Seconds::ZERO)
+            .disconnect_timeout(self.disconnect_timeout)
+            .await;
     }
 
     /// Run client and handle control messages
     pub async fn start<F, S>(self, service: F) -> Result<(), MqttError<Err>>
     where
-        F: IntoService<S> + 'static,
-        S: Service<Request = ControlMessage<Err>, Response = ControlResult, Error = Err>
-            + 'static,
+        F: IntoService<S, ControlMessage<Err>>,
+        S: Service<ControlMessage<Err>, Response = ControlResult, Error = Err> + 'static,
     {
         if self.keepalive.non_zero() {
             ntex::rt::spawn(keepalive(MqttSink::new(self.shared.clone()), self.keepalive));
@@ -267,23 +240,17 @@ where
             service.into_service(),
         );
 
-        Dispatcher::with(
-            self.io,
-            self.shared.state.clone(),
-            self.shared,
-            dispatcher,
-            Timer::new(Millis::ONE_SEC),
-        )
-        .keepalive_timeout(Seconds::ZERO)
-        .disconnect_timeout(self.disconnect_timeout)
-        .await
+        Dispatcher::new(self.io, self.shared, dispatcher)
+            .keepalive_timeout(Seconds::ZERO)
+            .disconnect_timeout(self.disconnect_timeout)
+            .await
     }
 }
 
 fn dispatch<Err, PErr>(
     router: Router<usize>,
     handlers: Vec<Handler<PErr>>,
-) -> impl Service<Request = Publish, Response = Either<Publish, PublishAck>, Error = Err>
+) -> impl Service<Publish, Response = Either<Publish, PublishAck>, Error = Err>
 where
     PErr: 'static,
     PublishAck: TryFrom<PErr, Error = Err>,
@@ -323,7 +290,7 @@ fn call<S, Err>(
     srv: &S,
 ) -> impl Future<Output = Result<Either<Publish, PublishAck>, Err>>
 where
-    S: Service<Request = Publish, Response = PublishAck>,
+    S: Service<Publish, Response = PublishAck>,
     PublishAck: TryFrom<S::Error, Error = Err>,
 {
     let fut = srv.call(req);
