@@ -1,9 +1,12 @@
+use std::io;
+
 use ntex::util::ByteString;
 
 use crate::{error, v5::codec};
 
 pub use crate::v5::control::{Closed, ControlResult, Disconnect, Error, ProtocolError};
 
+#[derive(Debug)]
 pub enum ControlMessage<E> {
     /// Unhandled publish packet
     Publish(Publish),
@@ -15,6 +18,8 @@ pub enum ControlMessage<E> {
     ProtocolError(ProtocolError),
     /// Connection closed
     Closed(Closed),
+    /// Peer is gone
+    PeerGone(PeerGone),
 }
 
 impl<E> ControlMessage<E> {
@@ -38,11 +43,16 @@ impl<E> ControlMessage<E> {
         ControlMessage::ProtocolError(ProtocolError::new(err))
     }
 
+    pub(super) fn peer_gone(err: Option<io::Error>) -> Self {
+        ControlMessage::PeerGone(PeerGone(err))
+    }
+
     pub fn disconnect(&self, pkt: codec::Disconnect) -> ControlResult {
         ControlResult { packet: Some(codec::Packet::Disconnect(pkt)), disconnect: true }
     }
 }
 
+#[derive(Debug)]
 pub struct Publish(codec::Publish);
 
 impl Publish {
@@ -93,3 +103,18 @@ impl Publish {
         }
     }
 }
+
+#[derive(Debug)]
+pub struct PeerGone(Option<io::Error>);
+
+impl PeerGone {
+    /// Returns error reference
+    pub fn error(&self) -> Option<&io::Error> {
+        self.0.as_ref()
+    }
+
+    /// Ack PeerGone message
+    pub fn ack(self) -> ControlResult {
+        ControlResult { packet: None, disconnect: true }
+    }
+}
